@@ -9,7 +9,7 @@
 //
 /**@name action_build.cpp - The build building action. */
 //
-//      (c) Copyright 1998-2007 by Lutz Sammer, Jimmy Salmon, and
+//      (c) Copyright 1998-2008 by Lutz Sammer, Jimmy Salmon, and
 //                                 Russell Smith
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: action_build.cpp 8964 2007-06-30 20:17:02Z jim4 $
 
 //@{
 
@@ -42,6 +41,7 @@
 #include "animation.h"
 #include "player.h"
 #include "unit.h"
+#include "unit_cache.h"
 #include "sound.h"
 #include "actions.h"
 #include "map.h"
@@ -154,11 +154,10 @@ static CUnit *CheckAlreadyBuilding(CUnit *unit, CUnitType *type, int x, int y)
 	CUnit *table[UnitMax];
 	int n;
 
-	n = UnitCacheOnTile(x, y, table);
+	n = UnitCache.Select(x, y, table, UnitMax);
 	for (int i = 0; i < n; ++i) {
 		if (!table[i]->Destroyed && table[i]->Type == type &&
-				(table[i]->Player == unit->Player || unit->IsAllied(table[i])) &&
-				table[i]->Orders[0]->Action == UnitActionBuilt) {
+				(table[i]->Player == unit->Player || unit->IsAllied(table[i]))) {
 			return table[i];
 		}
 	}
@@ -192,7 +191,12 @@ static CUnit *CheckCanBuild(CUnit *unit)
 	//
 	if ((ontop = CanBuildUnitType(unit, type, x, y, 1)) == NULL) {
 		if ((ontop = CheckAlreadyBuilding(unit, type, x, y)) != NULL) {
-			CommandRepair(unit, x, y, ontop, FlushCommands);
+			unit->Orders[0]->Init();
+			unit->Orders[0]->Action = UnitActionRepair;
+			unit->Orders[0]->Goal = ontop;
+			unit->Orders[0]->Goal->RefsIncrease();
+			unit->Orders[0]->Range = unit->Type->RepairRange;
+			unit->SubAction = 0;
 			return NULL;
 		}
 
