@@ -9,7 +9,7 @@
 //
 /**@name action_resource.cpp - The generic resource action. */
 //
-//      (c) Copyright 2001-2007 by Crestez Leonard and Jimmy Salmon
+//      (c) Copyright 2001-2008 by Crestez Leonard and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: action_resource.cpp 8743 2007-04-14 02:24:50Z jim4 $
 
 //@{
 
@@ -57,6 +56,9 @@
 #define SUB_START_GATHERING 55
 #define SUB_GATHER_RESOURCE 60
 
+int AlliedUnitRecyclingEfficiency[MaxCosts] = {0, 50};
+int EnemyUnitRecyclingEfficiency[MaxCosts] = {0, 40};
+
 /*----------------------------------------------------------------------------
 --  Functions
 ----------------------------------------------------------------------------*/
@@ -116,7 +118,7 @@ static bool StartGathering(CUnit *unit)
 		return false;
 	}
 
-	// FIXME: 0 can happen, if placed too near by map maker.
+	// 0 can happen, for example with flying units.
 	Assert(MapDistanceBetweenUnits(unit, goal) <= 1);
 
 	//
@@ -126,21 +128,6 @@ static bool StartGathering(CUnit *unit)
 		goal->X + (goal->Type->TileWidth - 1) / 2 - unit->X,
 		goal->Y + (goal->Type->TileHeight - 1) / 2 - unit->Y);
 
-	//
-	// If resource has too many harvesting it or is still under construction, wait!
-	//
-	if ((goal->Type->MaxOnBoard && goal->Data.Resource.Active >= goal->Type->MaxOnBoard) ||
-			goal->Orders[0]->Action == UnitActionBuilt) {
-		// FIXME: Determine somehow when the resource will be free to use
-		// FIXME: Could we somehow find another resource?
-		// However the CPU usage is really low (no pathfinding stuff).
-		unit->Wait = 10;
-		return false;
-	}
-
-	// Activate the resource
-	goal->Data.Resource.Active++;
-
 	return true;
 }
 
@@ -188,20 +175,53 @@ static void GatherResource(CUnit *unit)
 {
 	CUnit *source = unit->Orders[0]->Goal;
 	int amount[MaxCosts] = {1, 1};
-	bool visible = source->IsVisibleAsGoal(unit->Player);
+	bool visible = source->IsAliveOnMap();
 	int i;
+	const int totalEfficiency[MaxCosts] = {100, 100};
+	const int *efficiency = totalEfficiency;
 
 	AnimateActionHarvest(unit);
 
 	// Calculate how much we can harvest.
 	if (visible && UnitHoldsResources(source)) {
+		// Determine efficiency of recycling buildings
+		if (unit->Player == source->Player || unit->Player->IsAllied(source)) {
+			efficiency = AlliedUnitRecyclingEfficiency;
+		} else if (unit->Player->IsEnemy(source)) {
+			efficiency = EnemyUnitRecyclingEfficiency;
+		}
+
+		// Update ResourcesHeld based on HP
+		// Source might have been healed or attacked
+		// Is this the best place for this?
+		if (efficiency != totalEfficiency) {
+			int hp = source->Variable[HP_INDEX].Max *
+				source->ResourcesHeld[1] / source->Type->ProductionCosts[1];
+			if (hp != 0 && hp != source->Variable[HP_INDEX].Value) {
+				for (i = 0; i < MaxCosts; ++i) {
+					source->ResourcesHeld[i] = source->Type->ProductionCosts[i] *
+						source->Variable[HP_INDEX].Value / source->Variable[HP_INDEX].Max;
+				}
+			}
+		}
+
+		// Calculate new resource amounts
 		CalculateRequestedAmount(unit->Type, source->ResourcesHeld, amount);
 		for (i = 0; i < MaxCosts; ++i) {
 			unit->Player->ProductionRate[i] -= unit->Data.Harvest.CurrentProduction[i];
-			unit->Data.Harvest.CurrentProduction[i] = amount[i];
+			unit->Data.Harvest.CurrentProduction[i] = amount[i] * efficiency[i] / 100;
 			unit->Player->ProductionRate[i] += unit->Data.Harvest.CurrentProduction[i];
 			source->ResourcesHeld[i] -= amount[i];
 		}
+
+		// Recycling loses hit points
+		if (efficiency != totalEfficiency) {
+			source->Variable[HP_INDEX].Value = source->Variable[HP_INDEX].Max *
+				source->ResourcesHeld[1] / source->Type->ProductionCosts[1];
+			if (source->Variable[HP_INDEX].Value == 0 && source->ResourcesHeld[1] != 0) {
+				source->Variable[HP_INDEX].Value = 1;
+			}
+		}
 	}
 
 	//
@@ -220,8 +240,14 @@ static void GatherResource(CUnit *unit)
 			unit->Data.Harvest.CurrentProduction[i] = 0;
 		}
 
-		// Find a new resource to harvest
-		FindNewResource(unit);
+		if (unit->OrderCount == 1) {
+			// Find a new resource to harvest
+			FindNewResource(unit);
+		} else {
+			unit->Orders[0]->Goal->RefsDecrease();
+			unit->Orders[0]->Goal = NoUnitP;
+			unit->ClearAction();
+		}
 
 		// Don't destroy the resource twice.
 		// This only happens when it's empty.
@@ -256,7 +282,6 @@ static void ResourceGiveUp(CUnit *unit)
 void HandleActionResource(CUnit *unit)
 {
 	if (unit->Wait) {
-		// FIXME: show idle animation while we wait?
 		unit->Wait--;
 		return;
 	}
