@@ -9,7 +9,7 @@
 //
 /**@name actions.cpp - The actions. */
 //
-//      (c) Copyright 1998-2007 by Lutz Sammer, Russell Smith, and Jimmy Salmon
+//      (c) Copyright 1998-2008 by Lutz Sammer, Russell Smith, and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: actions.cpp 8750 2007-04-14 18:09:46Z jim4 $
 
 //@{
 
@@ -39,6 +38,7 @@
 #include <time.h>
 
 #include "stratagus.h"
+#include "version.h"
 #include "unittype.h"
 #include "animation.h"
 #include "player.h"
@@ -133,6 +133,11 @@ int UnitShowAnimationScaled(CUnit *unit, const CAnimation *anim, int scale)
 			case AnimationExactFrame:
 				unit->Frame = unit->Anim.Anim->D.Frame.Frame;
 				break;
+			case AnimationRandomFrame:
+				unit->Frame = unit->Anim.Anim->D.RandomFrame.MinFrame +
+					SyncRand() % (unit->Anim.Anim->D.RandomFrame.MaxFrame - unit->Anim.Anim->D.RandomFrame.MinFrame + 1);
+				UnitUpdateHeading(unit);
+				break;
 
 			case AnimationWait:
 				unit->Anim.Wait = unit->Anim.Anim->D.Wait.Wait << scale >> 8;
@@ -363,13 +368,6 @@ static void HandleUnitAction(CUnit *unit)
 			// Release pending references.
 			//
 			if (unit->Orders[0]->Goal) {
-				// If mining decrease the active count on the resource.
-				if (unit->Orders[0]->Action == UnitActionResource &&
-						unit->SubAction == 60) {
-					// FIXME: SUB_GATHER_RESOURCE ?
-					unit->Orders[0]->Goal->Data.Resource.Active--;
-					Assert(unit->Orders[0]->Goal->Data.Resource.Active >= 0);
-				}
 				// Still shouldn't have a reference unless attacking
 				Assert(!(unit->Orders[0]->Action == UnitActionStill && !unit->SubAction));
 				unit->Orders[0]->Goal->RefsDecrease();
@@ -392,6 +390,7 @@ static void HandleUnitAction(CUnit *unit)
 			// Note subaction 0 should reset.
 			//
 			unit->SubAction = unit->State = 0;
+			unit->Wait = 0;
 
 			if (IsOnlySelected(unit)) { // update display for new action
 				SelectedUnitChanged();
@@ -488,7 +487,7 @@ void UnitActions(void)
 			time_t now;
 			char buf[256];
 
-			sprintf(buf, "log_of_boswars_%d.log", ThisPlayer->Index);
+			sprintf_s(buf, sizeof(buf), "log_of_boswars_%d.log", ThisPlayer->Index);
 			logf = fopen(buf, "wb");
 			if (!logf) {
 				return;
