@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: edmap.cpp 8624 2007-03-16 05:02:34Z jim4 $
 
 //@{
 
@@ -652,7 +651,7 @@ void CEditor::CreateRandomMap(void) const
 {
 	int mz;
 
-	mz = Map.Info.MapWidth > Map.Info.MapHeight ? Map.Info.MapWidth : Map.Info.MapHeight;
+	mz = std::max(Map.Info.MapWidth, Map.Info.MapHeight);
 
 	// make water-base
 	TileFill(0, 0, WATER_TILE, mz * 3);
