@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: loadgame.cpp 9030 2007-08-04 17:09:35Z jim4 $
 
 //@{
 
@@ -53,12 +52,13 @@
 #include "trigger.h"
 #include "actions.h"
 #include "minimap.h"
-#include "commands.h"
+#include "replay.h"
 #include "sound.h"
 #include "sound_server.h"
 #include "font.h"
 #include "pathfinder.h"
 #include "spells.h"
+#include "particle.h"
 
 /*----------------------------------------------------------------------------
 --  Variables
@@ -99,10 +99,11 @@ void CleanModules(void)
 	CleanTilesets();
 	Map.Clean();
 	Map.CleanFogOfWar();
+	CParticleManager::exit();
 	CleanReplayLog();
 	CleanSpells();
 	FreeVisionTable();
-	FreeAStar();
+	FreePathfinder();
 }
 
 /**
@@ -153,14 +154,12 @@ void LoadModules(void)
 	LoadIcons();
 	LoadCursors();
 	UI.Load();
-#ifndef DYNAMIC_LOAD
 	LoadMissileSprites();
-#endif
 	LoadConstructions();
 	LoadDecorations();
 	LoadUnitTypes();
 
-	InitAStar();
+	InitPathfinder();
 
 	LoadUnitSounds();
 	MapUnitSounds();
