@@ -9,7 +9,7 @@
 //
 /**@name util.h - General utilities. */
 //
-//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2008 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ extern unsigned SyncRandSeed;               /// Sync random seed value
 
 extern void InitSyncRand(void);             /// Initialize the syncron rand
 extern int SyncRand(void);                  /// Syncron rand
+extern int SyncRand(int max);               /// Syncron rand
 
 /*----------------------------------------------------------------------------
 --  Math
@@ -53,13 +54,24 @@ extern long isqrt(long num);
 --  Strings
 ----------------------------------------------------------------------------*/
 
+#include <string.h>
+
 #if !defined(_MSC_VER) || _MSC_VER < 1400
 #define _TRUNCATE ((size_t)-1)
 extern unsigned int strcpy_s(char *dst, size_t dstsize, const char *src);
 extern unsigned int strncpy_s(char *dst, size_t dstsize, const char *src, size_t count);
 extern unsigned int strcat_s(char *dst, size_t dstsize, const char *src);
+extern int sprintf_s(char *dest, size_t destSize, const char *format, ...);
 #endif
 
+inline char *new_strdup(const char *str)
+{
+	int len = strlen(str) + 1;
+	char *newstr = new char[len];
+	strcpy_s(newstr, len, str);
+	return newstr;
+}
+
 	/// strdup + strcat
 extern char *strdcat(const char *l, const char *r);
 	/// strdup + strcat + strcat
