@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script_map.cpp 8874 2007-06-09 17:26:31Z jim4 $
 
 //@{
 
@@ -35,20 +34,19 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <stdlib.h>
 
 #include "stratagus.h"
 #include "unit.h"
-#include "unittype.h"
+#include "unit_cache.h"
 #include "script.h"
 #include "map.h"
 #include "tileset.h"
 #include "minimap.h"
-#include "actions.h"
-#include "results.h"
 #include "ui.h"
 #include "player.h"
 #include "iolib.h"
+#include "video.h"
+#include "version.h"
 
 /*----------------------------------------------------------------------------
 --  Functions
@@ -78,7 +76,7 @@ static int CclStratagusMap(lua_State *l)
 			char buf[32];
 
 			value = LuaToString(l, j + 1);
-			sprintf(buf, StratagusFormatString, StratagusFormatArgs(StratagusVersion));
+			sprintf_s(buf, sizeof(buf), StratagusFormatString, StratagusFormatArgs(StratagusVersion));
 			if (strcmp(buf, value)) {
 				fprintf(stderr, "Warning not saved with this version.\n");
 			}
@@ -114,7 +112,7 @@ static int CclStratagusMap(lua_State *l)
 					Map.Fields = new CMapField[Map.Info.MapWidth * Map.Info.MapHeight];
 					Map.Visible[0] = new unsigned[Map.Info.MapWidth * Map.Info.MapHeight / 2];
 					memset(Map.Visible[0], 0, Map.Info.MapWidth * Map.Info.MapHeight / 2 * sizeof(unsigned));
-					InitUnitCache();
+					UnitCache.Init(Map.Info.MapWidth, Map.Info.MapHeight);
 					// FIXME: this should be CreateMap or InitMap?
 				} else if (!strcmp(value, "fog-of-war")) {
 					Map.NoFogOfWar = false;
@@ -243,37 +241,6 @@ static int CclCenterMap(lua_State *l)
 	return 0;
 }
 
-/**
-**  Show Map Location
-**
-**  @param l  Lua state.
-*/
-static int CclShowMapLocation(lua_State *l)
-{
-	CUnit *target;
-	const char *unitname;
-
-	// Put a unit on map, use its properties, except for
-	// what is listed below
-
-	LuaCheckArgs(l, 4);
-	unitname = LuaToString(l, 5);
-	target = MakeUnit(UnitTypeByIdent(unitname), ThisPlayer);
-	if (target != NoUnitP) {
-		target->Orders[0]->Action = UnitActionStill;
-		target->Variable[HP_INDEX].Value = 0;
-		target->X = LuaToNumber(l, 1);
-		target->Y = LuaToNumber(l, 2);
-		target->TTL = GameCycle + LuaToNumber(l, 4);
-		target->CurrentSightRange = LuaToNumber(l, 3);
-		MapMarkUnitSight(target);
-	} else {
-		DebugPrint("Unable to allocate Unit");
-	}
-
-	return 0;
-}
-
 /**
 **  Set fog of war on/off.
 **
@@ -310,31 +277,6 @@ static int CclSetMinimapTerrain(lua_State *l)
 	return 0;
 }
 
-/**
-**  Fog of war opacity.
-**
-**  @param l  Lua state.
-*/
-static int CclSetFogOfWarOpacity(lua_State *l)
-{
-	int i;
-
-	LuaCheckArgs(l, 1);
-	i = LuaToNumber(l, 1);
-	if (i < 0 || i > 255) {
-		PrintFunction();
-		fprintf(stdout, "Opacity should be 0 - 255\n");
-		i = 128;
-	}
-	FogOfWarOpacity = i;
-
-	if (!CclInConfigFile) {
-		Map.Init();
-	}
-
-	return 0;
-}
-
 /**
 **  Define Fog graphics
 **
@@ -462,14 +404,12 @@ void MapCclRegister(void)
 	lua_register(Lua, "StratagusMap", CclStratagusMap);
 	lua_register(Lua, "RevealMap", CclRevealMap);
 	lua_register(Lua, "CenterMap", CclCenterMap);
-	lua_register(Lua, "ShowMapLocation", CclShowMapLocation);
 
 	lua_register(Lua, "SetFogOfWar", CclSetFogOfWar);
 	lua_register(Lua, "GetFogOfWar", CclGetFogOfWar);
 	lua_register(Lua, "SetMinimapTerrain", CclSetMinimapTerrain);
 
 	lua_register(Lua, "SetFogOfWarGraphics", CclSetFogOfWarGraphics);
-	lua_register(Lua, "SetFogOfWarOpacity", CclSetFogOfWarOpacity);
 
 	lua_register(Lua, "LoadTileModels", CclLoadTileModels);
 	lua_register(Lua, "DefinePlayerTypes", CclDefinePlayerTypes);
