@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: unitsound.cpp 8747 2007-04-14 04:22:35Z jim4 $
 
 //@{
 
@@ -97,7 +96,7 @@ void MapUnitSounds(void)
 		//
 		// Parse all units sounds.
 		//
-		for (std::vector<CUnitType *>::size_type i = 0; i < UnitTypes.size(); ++i) {
+		for (size_t i = 0; i < UnitTypes.size(); ++i) {
 			CUnitType *type = UnitTypes[i];
 
 			MapAnimSounds(type);
