@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: icons.cpp 8631 2007-03-22 19:13:27Z Nehal $
 
 //@{
 
@@ -143,7 +142,7 @@ void InitIcons(void)
 */
 void LoadIcons(void)
 {
-	for (std::vector<CIcon *>::size_type i = 0; i < AllIcons.size(); ++i) {
+	for (size_t i = 0; i < AllIcons.size(); ++i) {
 		CIcon *icon = AllIcons[i];
 		icon->G->Load();
 		ShowLoadProgress("Icons %s", icon->G->File.c_str());
