@@ -26,7 +26,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: ui.cpp 8839 2007-06-02 17:05:34Z jim4 $
 
 //@{
 
@@ -48,6 +47,7 @@
 #include "menus.h"
 #include "iolib.h"
 #include "unit.h"
+#include "title.h"
 
 /*----------------------------------------------------------------------------
 -- Variables
@@ -70,13 +70,14 @@ CUserInterface::CUserInterface() :
 	MouseScrollSpeedDefault(0), MouseScrollSpeedControl(0),
 	MouseWarpX(0), MouseWarpY(0),
 	SingleSelectedButton(NULL),
-	MaxSelectedFont(0), MaxSelectedTextX(0), MaxSelectedTextY(0),
+	MaxSelectedFont(NULL), MaxSelectedTextX(0), MaxSelectedTextY(0),
 	SingleTrainingButton(NULL),
-	SingleTrainingFont(0), SingleTrainingTextX(0), SingleTrainingTextY(0),
-	TrainingFont(0), TrainingTextX(0), TrainingTextY(0),
+	SingleTrainingFont(NULL), SingleTrainingTextX(0), SingleTrainingTextY(0),
+	TrainingFont(NULL), TrainingTextX(0), TrainingTextY(0),
 	CompletedBarColor(0), CompletedBarShadow(0),
 	ViewportMode(VIEWPORT_SINGLE), MouseViewport(NULL),
 	SelectedViewport(NULL), NumViewports(0),
+	MessageFont(NULL), MessageScrollSpeed(5),
 	ViewportCursorColor(0), Offset640X(0), Offset480Y(0),
 	VictoryBackgroundG(NULL), DefeatBackgroundG(NULL)
 {
@@ -157,22 +158,23 @@ void CUserInterface::Load(void)
 	//
 	for (i = 0; i < (int)Fillers.size(); ++i) {
 		Fillers[i].G->Load();
+		Fillers[i].G->UseDisplayFormat();
 	}
 
-	for (i = 0; i <= ScoreCost; ++i) {
+	for (i = 0; i < MaxCosts; ++i) {
 		if (Resources[i].G) {
 			Resources[i].G->Load();
+			Resources[i].G->UseDisplayFormat();
 		}
 	}
 
-	if (InfoPanel.G) {
-		InfoPanel.G->Load();
-	}
 	if (ButtonPanel.G) {
 		ButtonPanel.G->Load();
+		ButtonPanel.G->UseDisplayFormat();
 	}
 	if (PieMenu.G) {
 		PieMenu.G->Load();
+		PieMenu.G->UseDisplayFormat();
 	}
 
 	//
@@ -207,7 +209,6 @@ static void SaveViewports(CFile *file, const CUserInterface *ui)
 	int i;
 	const CViewport *vp;
 
-	// FIXME: don't save the number
 	file->printf("DefineViewports(\"mode\", %d", ui->ViewportMode);
 	for (i = 0; i < ui->NumViewports; ++i) {
 		vp = &ui->Viewports[i];
@@ -248,18 +249,10 @@ void CleanUserInterface(void)
 	UI.Fillers.clear();
 
 	// Resource Icons
-	for (i = 0; i <= ScoreCost; ++i) {
+	for (i = 0; i < MaxCosts; ++i) {
 		CGraphic::Free(UI.Resources[i].G);
 	}
 
-	// Info Panel
-	CGraphic::Free(UI.InfoPanel.G);
-	for (std::vector<CUnitInfoPanel *>::iterator panel = UI.InfoPanelContents.begin();
-			panel != UI.InfoPanelContents.end(); ++panel) {
-		delete *panel;
-	}
-	UI.InfoPanelContents.clear();
-
 	delete UI.SingleSelectedButton;
 	UI.SelectedButtons.clear();
 	delete UI.SingleTrainingButton;
