@@ -7,10 +7,12 @@ rm -rf boswars/engine
 rm -f boswars/bos.sln
 rm -f boswars/SConstruct
 
+wget -N http://www.boswars.org/dependencies/win32/windows-runtime-libs.zip
+unzip windows-runtime-libs.zip
+mv windows-runtime-libs/*.dll boswars/
+rmdir windows-runtime-libs
+
 cp engine/Release/boswars.exe boswars
-cp tools/installers/nsi/SDL.dll boswars
-cp tools/installers/nsi/msvcp80.dll boswars
-cp tools/installers/nsi/msvcr80.dll boswars
 cp tools/installers/nsi/Microsoft.VC80.CRT.manifest boswars
 
 echo "Creating installer"
